--imports
local assert = require("justo.assert")
local justo = require("justo")
local suite, test = justo.suite, justo.test

local function exec(cmd)
  local file = io.popen(cmd)
  local out = file:read("*all")
  local _, _, status = file:close()

  return status, out
end

--Suite.
return suite("dogmac check", function()
  test("dogmac check -h", function()
    local status, out = exec("dogmac -C check -h")
    assert(status):eq(0)
    assert(out):like("dogmac check")
  end)

  test("dogmac check", function()
    local status, out = exec("dogmac -C check")
    assert(status):eq(0)
    assert(out):eq("")
  end)

  suite("syn", function()
    test("file with errors", function()
      local status = exec("dogmac -C check test/data/check/invalid-syn.dog 2> /dev/null")
      assert(status):ne(0)
    end)

    test("correct file", function()
      local status = exec("dogmac -C check test/data/check/valid/Coord2D.dog")
      assert(status):eq(0)
    end)

    test("unknown file", function()
      local status = exec("dogmac -C check unknown.dog")
      assert(status):ne(0)
    end)

    test("with -l", function()
      local status = exec("dogmac -C check -l test/data/check/valid/Coord2D.dog")
      assert(status):eq(0)
    end)

    test("with -s", function()
      local status = exec("dogmac -C check -s test/data/check/valid/Coord2D.dog")
      assert(status):eq(0)
    end)

    test("with several files", function()
      local status = exec("dogmac -C check test/data/check/valid test/data/check/valid.dog")
      assert(status):eq(0)
    end)
  end)

  suite("lex", function()
    test("file with errors", function()
      local status = exec("dogmac -C check -t lex test/data/check/invalid-lex.dog 2> /dev/null")
      assert(status):ne(0)
    end)

    test("correct file", function()
      local status = exec("dogmac -C check -t lex test/data/check/valid/Coord2D.dog")
      assert(status):eq(0)
    end)

    test("unknown file", function()
      local status = exec("dogmac -C check -t lex unknown.dog")
      assert(status):ne(0)
    end)

    test("with -l", function()
      local status = exec("dogmac -C check -l -t lex test/data/check/valid/Coord2D.dog")
      assert(status):eq(0)
    end)

    test("with -s", function()
      local status = exec("dogmac -C check -s -t lex test/data/check/valid/Coord2D.dog")
      assert(status):eq(0)
    end)

    test("with several files", function()
      local status = exec("dogmac -C check -t lex test/data/check/valid test/data/check/valid.dog")
      assert(status):eq(0)
    end)
  end)
end)
